using System;
using System.Data;
using System.Web;
using System.Collections;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.ComponentModel;

using VaultPluginLib;
using VaultLib;
using VaultClientIntegrationLib;
using VaultClientOperationsLib;
using VaultClientNetLib.ClientService;

using System.Xml;
using System.Xml.Serialization;
using System.Resources;
using System.Reflection;
namespace VaultPluginTemplate
{
    /// <summary>
    /// Summary description for PluginServiceTemplate
    /// </summary>
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    /// <summary>
    /// Summary description for PluginServiceTemplate.
    /// </summary>
    [WebService(Namespace = "http://www.sourcegear.com/schemas/vault", Description = "A sample plugin to demonstrate handling Vault events.  In order to configure this plugin, you must invoke <a href=PluginServiceTemplate.asmx?op=UpdateVaultConnectionInformation>UpdateVaultConnectionInformation</a>, then <a href=PluginServiceTemplate.asmx?op=UpdateRepositoryInformation>UpdateRepositoryInformation</a>, and then <a href=PluginServiceTemplate.asmx?op=RegisterWithServer>RegisterWithServer</a> ")]
    public class PluginServiceTemplate : VaultPluginServiceBase
    {
        public string WebConfigLoginKey = "vaultlogin";
        public string WebConfigPasswordKey = "vaultpassword";
        public string WebConfigServerKey = "vaultserver";
        public string WebConfigRepIDKey = "repidtowatch";
        public string WebConfigPathKey = "pathtowatch";
        public string WebConfigLastVersionKey = "lastversion";

        public PluginServiceTemplate()
        {
            //CODEGEN: This call is required by the ASP.NET Web Services Designer
            InitializeComponent();
        }

        #region Component Designer generated code

        //Required by the Web Services Designer 
        private IContainer components = null;

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing && components != null)
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        #endregion

        /// <summary>
        /// Overrides the base class definition of OnEndTx.  Allows us to do something
        /// when we catch and EndTx event.
        /// </summary>
        /// <param name="nRepID">The repository that was modified.</param>
        /// <param name="txID">The new transactionID that resulted from the modification.</param>
        /// <returns></returns>
        protected override int OnEndTx(int nRepID, long txID)
        {

            int watchrepid = int.Parse((string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigRepIDKey]);
            if (watchrepid != nRepID)
                return 1; // We only care about modifications to the one repository

            string username = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigLoginKey];
            string password = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigPasswordKey];
            string hostname = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigServerKey];
            string pathToWatch = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigPathKey];

            long lastversion = 0;
            try
            {
                lastversion = long.Parse((string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigLastVersionKey]);
            }
            catch (Exception)
            {
                lastversion = 0;
            }

            #region log in to the Vault server
            //It would be kind to the Vault server to have one client instance that
            //stayed around and just did a refresh for each EndTx event.  
            ServerOperations.client.LoginOptions.URL = "http://" + hostname + "/VaultService";
            ServerOperations.client.LoginOptions.User = username;
            ServerOperations.client.LoginOptions.Password = password;
            ServerOperations.Login();
            FindRepositoryByID(ServerOperations.client.ClientInstance, nRepID);
            #endregion

            VaultClientFolder vcfolder = ServerOperations.client.ClientInstance.TreeCache.Repository.Root.FindFolderRecursive(pathToWatch);
            long currentversion = 0;
            if (vcfolder == null)
            {
                VaultClientFile vcfile = ServerOperations.client.ClientInstance.TreeCache.Repository.Root.FindFileRecursive(pathToWatch);
                if (vcfile != null)
                    currentversion = vcfile.Version;
            }
            else
                currentversion = vcfolder.Version;


            if (currentversion != lastversion)
            {
                ArrayList arrHistoryItems = new ArrayList();
                LoadHistory(ServerOperations.client.ClientInstance, pathToWatch, txID, ref arrHistoryItems);

                foreach (VaultHistoryItem item in arrHistoryItems)
                {
                    //If this were going to do something, this is the place to do it.
                    //The things of interest inside item are:
                    //item.Name -- The path to the item that was modified
                    //item.MiscInfo1 -- An english description of what happened.  This 
                    //					is generated by the LoadHistory method, so you 
                    //					can change it if you want.

                }
                string strServerPath = Server.MapPath(Context.Request.ApplicationPath);
                WebConfigurator.SetWebConfigSetting(strServerPath, WebConfigLastVersionKey, currentversion.ToString());
            }
            ServerOperations.Logout();
            return 1;
        }

        /// <summary>
        /// Loads the history items that occurred in the given transactionID and stores them in the
        /// given ArrayList.  Note that each item will have MiscInfo1 set to an english description of the
        /// history event.
        /// </summary>
        public static void LoadHistory(ClientInstance ciUser, string repositoryPath, long ntxID, ref ArrayList arrHistoryItems)
        {
            arrHistoryItems.AddRange(ServerOperations.ProcessCommandHistory(repositoryPath, true, DateSortOption.desc, "", "", DateTime.Now.AddMinutes(-30).ToString(), DateTime.Now.ToString(), "", "", -1, -1, 1000));
            /*
            #region download the history, based on a query that gets all the history for the last 30 minutes.
            VaultLib.VaultHistoryQueryRequest req = new VaultLib.VaultHistoryQueryRequest();
            DateTime now = DateTime.Now;
            ArrayList users = new ArrayList();
            ArrayList substrings = new ArrayList();
            ArrayList actions = new ArrayList();
            ArrayList sorts = new ArrayList();
            sorts.Add((long)VaultQueryRequestSort.ActionSort);
            req.SubstringType = VaultLib.VaultQueryRequestSubstrings.DoNotFilter;
            req.IsFolder = true;
            req.Recursive = true;

            req.DateFilterType = VaultQueryRequestDateTypes.IncludeRange;
            req.DateFilterMask = VaultQueryRequestDates.HistoryBefore | VaultQueryRequestDates.HistoryAfter;
            req.EndDate = VaultDateTime.Now;
            //This is kind of a hack.  I load all of the history for
            //the last 30 minutes and then select out the events I'm
            //interested in by txID.
            req.BeginDate = req.EndDate.AddSeconds(-30 * 60);

            actions.Clear();
            req.CommentFilter = VaultLib.VaultQueryRequestComments.DoNotFilter;

            req.RepID = ciUser.ActiveRepositoryID;
            req.TopName = repositoryPath;

            req.Recursive = true;
            req.Users = (VaultLib.VaultUser[])users.ToArray(typeof(VaultLib.VaultUser));
            req.Substrings = (string[])substrings.ToArray(typeof(string));
            req.Actions = (long[])actions.ToArray(typeof(long));
            req.Sorts = (long[])sorts.ToArray(typeof(long));

            string token = String.Empty;
            int rowsRetrieved = 0;
            VaultLib.VaultHistoryItem[] items = new VaultLib.VaultHistoryItem[1];
            ciUser.Connection.HistoryBegin(req, 100, ref rowsRetrieved, ref token);
            if (rowsRetrieved > 0)
            {
                ciUser.Connection.HistoryFetch(token, 0, rowsRetrieved - 1, ref items);
            }
            ciUser.Connection.HistoryEnd(token);
            #endregion

            string strFormat = "On {0}, User {1} performed the action \"{2}\" on {3} with comment \"{4}\"";
            if (rowsRetrieved > 0)
            {
                ResourceManager rm = new ResourceManager("VaultClientPresentationLib.Resources.HistoryExplorerStrings", Assembly.Load("VaultPresentationLib"));
                foreach (VaultLib.VaultHistoryItem item in items)
                {
                    if (item.TxID == ntxID)
                    {
                        string strAction;
                        #region translate histitemtype to a string that makes sense.
                        switch (item.HistItemType)
                        {
                            case VaultHistoryType.Added:
                                strAction = String.Format("{0} {1}", rm.GetString("ActionAdded"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.BranchedFrom:
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionBranchedItem"), item.MiscInfo2, rm.GetString("From"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.BranchedFromShare:
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionBranchedItem"), item.MiscInfo1, rm.GetString("From"), rm.GetString("Share"));
                                break;
                            case VaultHistoryType.BranchedFromItem:
                                strAction = String.Format("{0} {1} {2}", rm.GetString("ActionBranchedItem"), rm.GetString("From"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.BranchedFromShareItem:
                                strAction = String.Format("{0} {1} {2}", rm.GetString("ActionBranchedItem"), rm.GetString("From"), rm.GetString("Share"));
                                break;
                            case VaultHistoryType.CheckIn:
                                strAction = rm.GetString("ActionCheckIn");
                                break;
                            case VaultHistoryType.Created:
                                strAction = rm.GetString("ActionCreated");
                                break;
                            case VaultHistoryType.Deleted:
                                strAction = String.Format("{0} {1}", rm.GetString("ActionDeleted"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.Label:
                                strAction = String.Format("{0} {1} ", rm.GetString("ActionLabeled"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.MovedFrom:
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionMovedFrom"), item.MiscInfo1, rm.GetString("From"), item.MiscInfo2);
                                break;
                            case VaultHistoryType.MovedTo:
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionMovedTo"), item.MiscInfo1, rm.GetString("To"), item.MiscInfo2);
                                break;
                            case VaultHistoryType.Obliterated:
                                strAction = String.Format("{0} {1}", rm.GetString("ActionObliterated"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.Pinned:
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionPinned"), item.MiscInfo1, rm.GetString("AtVersion"), item.MiscInfo2);
                                break;
                            case VaultHistoryType.PropertyChange:
                                //we get this in the form E:x,M:y where E: marks the change for eol id, and M: marks the change for mergeable. 
                                //x will be the new value of the eolid and y will be the new value of the mergeable setting.
                                //x or y of 0 means no change and -1 means the value wasn't exp
                                try
                                {
                                    string s = item.MiscInfo1;
                                    string[] props = item.MiscInfo1.Split(',');
                                    string propName = string.Empty;
                                    string propValue = string.Empty;
                                    string[] propNames = null;
                                    string[] propValues = null;
                                    strAction = "";

                                    if (props != null)
                                    {
                                        propNames = new string[props.Length];
                                        propValues = new string[props.Length];
                                        for (int j = 0; j < props.Length; j++)
                                        {
                                            string[] tmp = props[j].Split(':');
                                            if (int.Parse(tmp[1]) == -100)
                                            {
                                                propNames[j] = null;
                                                propValues[j] = null;
                                            }
                                            else
                                            {
                                                if (tmp[0] == "E")
                                                {
                                                    propNames[j] = rm.GetString("EOLConversion");
                                                    switch (int.Parse(tmp[1]))
                                                    {
                                                        case VaultLib.VaultEOLForm.CR:
                                                            propValues[j] = rm.GetString("CR");
                                                            break;
                                                        case VaultLib.VaultEOLForm.CRLF:
                                                            propValues[j] = rm.GetString("CRLF");
                                                            break;
                                                        case VaultLib.VaultEOLForm.LF:
                                                            propValues[j] = rm.GetString("LF");
                                                            break;
                                                        case VaultLib.VaultEOLForm.Native:
                                                            propValues[j] = rm.GetString("Native");
                                                            break;
                                                        case VaultLib.VaultEOLForm.None:
                                                            propValues[j] = rm.GetString("None");
                                                            break;
                                                    }
                                                }

                                                else if (tmp[0] == "M")
                                                {
                                                    propNames[j] = rm.GetString("FileType");
                                                    switch (int.Parse(tmp[1]))
                                                    {
                                                        case VaultLib.VaultMergeable.Binary:
                                                            propValues[j] = rm.GetString("Binary");
                                                            break;
                                                        case VaultLib.VaultMergeable.Mergeable:
                                                            propValues[j] = rm.GetString("Mergeable");
                                                            break;
                                                        case VaultLib.VaultMergeable.UseSystemDefault:
                                                            propValues[j] = rm.GetString("UseSystemDefault");
                                                            break;
                                                    }
                                                }
                                            }
                                        }
                                    }

                                    for (int n = 0; n < propNames.Length; n++)
                                    {
                                        if (propNames[n] != null)
                                        {
                                            if (strAction.Length > 0)
                                                strAction += ", ";

                                            strAction += string.Format("{0} {1} {2}", propNames[n], rm.GetString("changedTo"), propValues[n]);
                                        }
                                    }
                                }
                                catch
                                {
                                    strAction = rm.GetString("PropertyChanged");
                                }

                                break;
                            case VaultHistoryType.Renamed:
                                strAction = String.Format("{0} {1} {2} {3} {4}", rm.GetString("ActionRenamedItem"), rm.GetString("From"), item.MiscInfo2, rm.GetString("To"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.RenamedItem:
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionRenamed"), item.MiscInfo2, rm.GetString("To"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.Rollback:
                                //strAction = rm.GetString("ActionRollback");
                                strAction = String.Format("{0} {1} {2}", rm.GetString("ActionRollback"), rm.GetString("ToVersion"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.SharedTo:
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionShared"), item.MiscInfo1, rm.GetString("As"), item.MiscInfo2);
                                break;
                            case VaultHistoryType.Undeleted:
                                strAction = String.Format("{0} {1}", rm.GetString("ActionUndeleted"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.UnPinned:
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionUnPinned"), item.MiscInfo1, rm.GetString("FromVersion"), item.MiscInfo2);
                                break;
                            case VaultHistoryType.Snapshot:
                                strAction = String.Format("{0} {1} ", rm.GetString("ActionSnapshot"), item.MiscInfo1);
                                break;
                            case VaultHistoryType.SnapshotFrom:
                                string snapshotName = string.Empty;
                                //display only the name of the labeled folder, not the full path
                                if (item.MiscInfo2 != null)
                                {
                                    string[] pathItems = item.MiscInfo2.Split('/');
                                    snapshotName = pathItems[pathItems.Length - 1];
                                }
                                strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionSnapshot"), item.MiscInfo1, rm.GetString("As"), snapshotName);
                                break;
                            case VaultHistoryType.SnapshotItem:
                                strAction = String.Format("{0} {1} {2} ", rm.GetString("ActionSnapshot"), rm.GetString("From"), item.MiscInfo1);
                                break;
                            default:
                                strAction = rm.GetString("ActionUnknown");
                                break;
                        }
                        #endregion
                        string strtmp = String.Format(strFormat, item.TxDate, item.UserLogin, strAction, item.Name, item.Comment);
                        ///Another sorta-hack.  I'm too lazy to create a separate array
                        ///to hold the english description, so I just put it back into MiscInfo1,
                        ///which is used to create the description.
                        item.MiscInfo1 = strtmp;
                        arrHistoryItems.Add(item);
                    }
             
                }
            }
             */
        }

        [WebMethod(Description = "Registers this plugin with the server that is set up by calling <a href=PluginServiceTemplate.asmx?op=UpdateVaultConnectionInformation>UpdateVaultConnectionInformation</a>.  You must have the password for the \"admin\" account on the server to register the plugin.")]
        public string RegisterWithServer(string adminPassword)
        {
            string serverHostname = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigServerKey];

            if (serverHostname.StartsWith("http://") == false)
                serverHostname = "http://" + serverHostname;
            if (serverHostname.EndsWith("/VaultService") == false)
                serverHostname += "/VaultService";
            ClientInstance myClient = null;
            try
            {
                myClient = new ClientInstance();
                myClient.Init(VaultClientNetLib.VaultConnection.AccessLevelType.Admin);
                myClient.Login(serverHostname, "admin", adminPassword);
            }
            catch (Exception e)
            {
                return "Could not connect to server.  Got exception " + e.Message;
            }

            //Build the array of events that this plugin accepts, based on which methods this class overrides
            #region build array of supported methods
            ArrayList alNotifyEvents = new ArrayList(4);
            Type myType = this.GetType();
            MethodInfo[] miarr = myType.GetMethods(BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly);
            foreach (MethodInfo mi in miarr)
            {
                if (mi.DeclaringType == myType && mi.Name == "OnEndTx")
                {
                    alNotifyEvents.Add(VaultLib.VaultPluginEvent.EndTxEvent);
                }
                else if (mi.DeclaringType == myType && mi.Name == "OnBeginTx")
                {
                    alNotifyEvents.Add(VaultLib.VaultPluginEvent.BeginTxEvent);
                }
                else if (mi.DeclaringType == myType && mi.Name == "OnLogin")
                {
                    alNotifyEvents.Add(VaultLib.VaultPluginEvent.LoginEvent);
                }
                else if (mi.DeclaringType == myType && mi.Name == "OnLogout")
                {
                    alNotifyEvents.Add(VaultLib.VaultPluginEvent.LogoutEvent);
                }
            }
            #endregion

            string currentURL = "http://" + Context.ApplicationInstance.Server.MachineName + Context.ApplicationInstance.Request.FilePath;
            myClient.Connection.RegisterPluginNotification(currentURL, Context.ApplicationInstance.Request.ApplicationPath, (int[])alNotifyEvents.ToArray(typeof(int)));
            myClient.Logout();
            return "Successfully Registered with the Vault Server at " + serverHostname;
        }

        [WebMethod(Description = "Unregisters this plugin with the server that is set up by calling <a href=PluginServiceTemplate.asmx?op=UpdateVaultConnectionInformation>UpdateVaultConnectionInformation</a>.  You must have the password for the \"admin\" account on the server to unregister the plugin.")]
        public string UnRegisterWithServer(string adminPassword)
        {
            string serverHostname = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigServerKey];
            if (serverHostname.StartsWith("http://") == false)
                serverHostname = "http://" + serverHostname;
            if (serverHostname.EndsWith("/VaultService") == false)
                serverHostname += "/VaultService";
            ClientInstance myClient = null;
            try
            {
                myClient = new ClientInstance();
                myClient.Init(VaultClientNetLib.VaultConnection.AccessLevelType.Admin);
                myClient.Login(serverHostname, "admin", adminPassword);
            }
            catch (Exception e)
            {
                return "Could not connect to server.  Got exception " + e.Message;
            }
            ArrayList alNotifyEvents = new ArrayList(4);
            alNotifyEvents.Add(VaultLib.VaultPluginEvent.EndTxEvent);
            alNotifyEvents.Add(VaultLib.VaultPluginEvent.BeginTxEvent);
            alNotifyEvents.Add(VaultLib.VaultPluginEvent.LoginEvent);
            alNotifyEvents.Add(VaultLib.VaultPluginEvent.LogoutEvent);

            string currentURL = "http://" + Context.ApplicationInstance.Server.MachineName + Context.ApplicationInstance.Request.FilePath;
            myClient.Connection.UnregisterPluginNotification(currentURL, Context.ApplicationInstance.Request.ApplicationPath, (int[])alNotifyEvents.ToArray(typeof(int)));
            myClient.Logout();
            return "Successfully unregistered with the Vault Server at " + serverHostname;
        }

        [WebMethod(Description = "Updates the information for which repository and path will be watched.")]
        public string UpdateRepositoryInformation(string repositoryName, string pathToWatch)
        {
            string strServerPath = Server.MapPath(Context.Request.ApplicationPath);
            string username = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigLoginKey];
            string password = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigPasswordKey];
            string hostname = (string)System.Configuration.ConfigurationSettings.AppSettings[WebConfigServerKey];

            ServerOperations.client.LoginOptions.URL = "http://" + hostname + "/VaultService";
            ServerOperations.client.LoginOptions.User = username;
            ServerOperations.client.LoginOptions.Password = password;

            ServerOperations.Login();
            //FindRepositoryByID(ServerOperations.client.ClientInstance, nRepID);
            VaultRepositoryInfo[] reps = null;
            //List all the repositories on the server.
            ServerOperations.client.ClientInstance.ListRepositories(ref reps);

            //Search for the one that we want.
            foreach (VaultRepositoryInfo r in reps)
            {
                if (r.RepName == repositoryName)
                {
                    //This will load up the client side cache files and refresh the repository structure.
                    //See http://support.sourcegear.com/viewtopic.php?t=6 for more on client side cache files.
                    ServerOperations.client.ClientInstance.SetActiveRepositoryID(r.RepID, ServerOperations.client.ClientInstance.Connection.UserLogin, r.UniqueRepID, true, true);
                    break;
                }
            }
            if (ServerOperations.client.ClientInstance.ActiveRepositoryID == -1)
                throw new Exception(string.Format("Repository {0} not found", repositoryName));

            if (null == ServerOperations.client.ClientInstance.TreeCache.Repository.Root.FindFolderRecursive(pathToWatch)
                && null == ServerOperations.client.ClientInstance.TreeCache.Repository.Root.FindFileRecursive(pathToWatch))
                return "Could not find path " + pathToWatch + " in repository " + repositoryName;

            WebConfigurator.SetWebConfigSetting(strServerPath, WebConfigRepIDKey, ServerOperations.client.ClientInstance.ActiveRepositoryID.ToString());
            WebConfigurator.SetWebConfigSetting(strServerPath, WebConfigPathKey, pathToWatch);
            return "Now watching repository " + repositoryName + " for updates to " + pathToWatch;
        }
        /// <summary>
        /// Updates the Vault connection information used to get infomation about the transaction that completed.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Updates the Vault connection information used to get infomation about the transaction that completed.")]
        public string UpdateVaultConnectionInformation(string server, string login, string password)
        {
            //	Write to config file if not already written
            string strServerPath = Server.MapPath(Context.Request.ApplicationPath);

            WebConfigurator.SetWebConfigSetting(strServerPath, WebConfigLoginKey, login);
            //Yes, I'm aware that this is writing the password in plain text.  Feel free to compile your
            //username and password into your plugin or write your own crypto code.
            WebConfigurator.SetWebConfigSetting(strServerPath, WebConfigPasswordKey, password);
            WebConfigurator.SetWebConfigSetting(strServerPath, WebConfigServerKey, server);

            return "Successfully updated Server, User, and Password.";
        }

        /// <summary>
        /// This is a helper function that works around a deficiency in the 
        /// Vault Client API.  In the API, there is no method that lets you set the active repository
        /// for a ClientInstance based only on the repository name.  This means that (for now) every
        /// program that uses the API will need to do something like this.
        /// </summary>
        public static void FindRepositoryByID(ClientInstance client, int nRepID)
        {
            VaultRepositoryInfo[] reps = null;
            //List all the repositories on the server.
            client.ListRepositories(ref reps);

            //Search for the one that we want.
            foreach (VaultRepositoryInfo r in reps)
            {
                if (r.RepID == nRepID)
                {
                    //This will load up the client side cache files and refresh the repository structure.
                    //See http://support.sourcegear.com/viewtopic.php?t=6 for more on client side cache files.
                    client.SetActiveRepositoryID(r.RepID, client.Connection.UserLogin, r.UniqueRepID, true, true);
                    break;
                }
            }
            if (client.ActiveRepositoryID == -1)
                throw new Exception(string.Format("Repository {0} not found", nRepID));
        }
    }
}
